#include <iostream>
#include <fstream>
#include <random>
#include <cassert>
#include <cstdlib>
#include <ctime>
using namespace std;

mt19937 rg(time(0));

int config[15][2] = {
/* #01 (1)*/ {3, 30},
/* #02 (1)*/ {7, 60},
/* #03 (1)*/ {10, 100},
/* #04 (1)*/ {15, 999},
/* #05 (2)*/ {19, 78},
/* #06 (2)*/ {20, 87},
/* #07 (2)*/ {26, 99}, /*/
/* #08 (2)*/ {30, 99},
/* #09 (3)*/ {35, 49}, /*/
/* #10 (3)*/ {40, 99},
/* #11 (3)*/ {45, 99},
/* #12 (3)*/ {50, 99},
/* #13 (4)*/ {70, 777},
/* #14 (4)*/ {95, 888},
/* #15 (4)*/ {100, 999}
};

// constraints:
// 1) N <= 15 - 4 tests
// 2) N <= 30 - 8 tests
// 3) N <= 50 && S <= 100 - 11 tests

int random(int from, int to)
{
    return from + rg() % (to - from + 1);
}

vector<int> partitionSum(int n, int s, bool large = false)
{
    if (n == 1) return {s};

    vector<int> v;

    if (large)
    {
        int x = random(s / 6, s / 2);
        v.push_back(x);
        n--;
        s -= x;
    }

    while (n > 1)
    {
        int x = random(1, 2 * s / n);
        if (s - x < n - 1) x = 1;
        v.push_back(x);
        s -= x;
        n--;
    }
    v.push_back(s);

    return v;
}

int main()
{
    for (int i = 1; i <= 15; ++ i)
    {
        int n = config[i - 1][0], s = config[i - 1][1];

        cout << "n = " << n << ", s = " << s << endl;

        int s1 = s / 3, s2 = s / 3, s3 = s - s1 - s2;
        vector<int> v;

        if (i == 7 or i == 9) // no guarantee for solution
        {
            v = partitionSum(n, s, i == 7);
            if (i == 9)
            {
                for (int j = 0; j < v.size(); ++ j)
                {
                    v[j] *= 2;
                }
                v.push_back(1);
                s++;
                n++;
            }
        }
        else // guarantee a solution
        {
            vector<int> ns = partitionSum(3, n);
            cout << "ns = [" << ns[0] << ", " << ns[1] << ", " << ns[2] << "]" << endl;
            vector<int> v1 = partitionSum(ns[0], s1, rg() % 7 == 0 && ns[0] < 2 * s1);
            vector<int> v2 = partitionSum(ns[1], s2, rg() % 7 == 0 && ns[1] < 2 * s2);
            vector<int> v3 = partitionSum(ns[2], s3, rg() % 7 == 0 && ns[2] < 2 * s3);
            for (int e : v1) v.push_back(e);
            for (int e : v2) v.push_back(e);
            for (int e : v3) v.push_back(e);
        }

        assert(v.size() == n);

        string no = "  ";
        no[0] = i / 10 + 48;
        no[1] = i % 10 + 48;
        string filename = "three." + no + ".in";

        ofstream test(filename);
        test << n << endl;
        for (int j = 0; j < v.size(); ++ j)
        {
            if (i == 13) test << 1 << ' ' << v[j] << endl;
            else if (i % 3 == 0) test << rg() % 3 + 1 << ' ' << v[j] << endl;
            else
            {
                int rn = rg() % 6;
                if (rn < 1) test << 1 << ' ' << v[j] << endl;
                else if (rn < 3) test << 2 << ' ' << v[j] << endl;
                else test << 3 << ' ' << v[j] << endl;
            }
        }
        test.close();

        string command = "..\\solutions\\three-author.exe < " + filename + " > three." + no + ".sol";
        cout << command << endl;
        system(command.c_str());
    }

    return 0;
}
